#!/usr/bin/env python
# encoding: utf-8
"""
abyss_helper_functions.py

Created by Rod Docking on 2010-11-05.
Copyright (c) 2010 Canada's Michael Smith Genome Sciences Centre. All rights reserved.
"""

import sys
import os
import string

def reverse_complement(seq):
    """Return the reverse complement"""
    
    return seq.translate(string.maketrans("ATCGNatcgn", "TAGCNtagcn"))[::-1]

def reverse_contig_path(path):
    """From an ABySS-style contig path list, return the reversed orientation"""
    
    flip_signs = [p.translate(string.maketrans("+-", "-+")) for p in path]
    return flip_signs[::-1]

def reverse_contig_path_string(path_string, delimiter=","):
    
    path_list = path_string.split(delimiter)
    reverse_list = reverse_contig_path(path_list)
    return ",".join(reverse_list)

def reverse_node_ori(node):
    """Just flip +/- in a node name"""
    return node.translate(string.maketrans("+-", "-+"))

def merge_contigs(path, se_contig_seq_dict, kmer_length):
    """From a contig path, construct the new contig sequence"""
    
    #print "Merging path:", path
    
    # Get the seed contig
    if "N" in path[0]:
        seq = "N" * int(path[0][:-1])
    else:    
        seq = se_contig_seq_dict[int(path[0].rstrip('+-'))]
        if '-' in path[0]:
            seq = reverse_complement(seq)
            
    for contig in path[1:]:
        
        if "N" in contig:
            seq = seq + "N" * int(contig[:-1])
        else:        
            seq_to_add = se_contig_seq_dict[int(contig.rstrip('+-'))]
            if '-' in contig:
                seq_to_add = reverse_complement(seq_to_add)
                
            if seq[-1] in "ACGT":
                
                # Confirm k-1 overlap
                # NOTE - using lower() for this comparison b/c of lower-case bases in some contigs
                #   when we write the merged contig, which one to use?
                try:
                    assert(seq[-(kmer_length-1):].lower() == seq_to_add[:(kmer_length-1)].lower())
                except AssertionError, e:
                    print "Assertion failed!!!!!! - Continuing, but check these sequences!"
                    print seq[-(kmer_length-1):].lower()
                    print seq_to_add[:(kmer_length-1)].lower()
                    
                seq = seq + seq_to_add[(kmer_length-1):]
                
            elif seq[-1] == "N":
                seq = seq + seq_to_add
                
    return seq

def main():
    pass


if __name__ == '__main__':
    main()

