#!/usr/bin/env python
# encoding: utf-8
"""
anchor_divert_static_alignments.py

Created by Rod Docking on 2011-03-25.
Copyright (c) 2011 Canada's Michael Smith Genome Sciences Centre. All rights reserved.
"""

import sys
import os
import subprocess

from SAM_record import SAM_record, bam_open

def main():
    """ From a list of contigs that are still being edited, and a BAM file,
        write four output files:
            - All read pairs where both are aligned as a proper pair to a 'static' BAM file
            - All other read pairs to:
                - A BAM file
                - First-read and second-read fastq files
    """
     
    # Read the list of edited contigs into a dictionary
    edit_dict = {}
    with open(sys.argv[1]) as edit_list:
        for line in edit_list:
            edit_dict[line.rstrip()] = 1

    # Open process for reading BAM file
    input_proc = subprocess.Popen(['samtools', 'view', '-h', sys.argv[2]],
                                    shell=False, cwd=os.getcwd(), 
                                    stdout=subprocess.PIPE)
    
    # Open processes for writing BAM files
    # static_proc = subprocess.Popen(['samtools', 'view', '-bS', '-'],
    #                                 shell=False, cwd=os.getcwd(), 
    #                                 stdin=subprocess.PIPE, stdout=open(sys.argv[3], 'wb'))
    
    realign_proc = subprocess.Popen(['samtools', 'view', '-bS', '-'],
                                    shell=False, cwd=os.getcwd(), 
                                    stdin=subprocess.PIPE, stdout=open(sys.argv[4], 'wb'))
     
    # Read pairs of alignments
    read_dict = {}
    for line in input_proc.stdout:
         
        # Output header lines to both output files
        S = SAM_record(line)
        if S.header:
            realign_proc.stdin.write(line)
            #static_proc.stdin.write(line)
            continue

        # If we have both ends of a pair, output reads
        if S.qname in read_dict:
            R = read_dict[S.qname]
            
            if (S.rname == R.rname and
                not S.rname in edit_dict and
                S.proper_pair()):
                continue
                #static_proc.stdin.write(S.raw_sam)
                #static_proc.stdin.write(R.raw_sam)
            else:
                realign_proc.stdin.write(S.raw_sam)
                realign_proc.stdin.write(R.raw_sam)
            
            del read_dict[S.qname]

        # Otherwise wait for the mate
        else:
            read_dict[S.qname] = S
                
if __name__ == '__main__':
    main()

