#!/usr/bin/env python
# encoding: utf-8
"""
infer_isize_and_read_length.py

Created by Rod Docking on 2011-03-29.
Copyright (c) 2011 Canada's Michael Smith Genome Sciences Centre. All rights reserved.
"""

import sys
import os

from SAM_record import SAM_record, bam_open

def main():
    """ From a BAM file, infer the average insert size and read length by
        looking at the first 100000 properly paired reads 
    """

    read_length, isize_list = None, []

    for line in bam_open(sys.argv[1]):
        
        if len(isize_list) >= 100000:
            break
        
        S = SAM_record(line)
        if S.header: continue
        
        if not read_length:
            read_length = S.read_length
        else:
            if S.read_length != read_length:
                print "Warning! Unequal read lengths: ", read_length, S.read_length    
        
        if S.proper_pair() and (not S.isize == 0):
            isize_list.append(abs(S.isize))

    midpoint = len(isize_list)/2
    sorted_isize_list = sorted(isize_list)
    median_isize = sorted_isize_list[midpoint]
    
    out_handle = open(sys.argv[2], 'w')
    out_handle.write("{0}\t{1}\n".format(read_length, median_isize))
    out_handle.close()

if __name__ == '__main__':
    main()

