#!/usr/bin/env python
# encoding: utf-8
"""
psl_record.py

Created by Rod Docking on 2010-10-20.
Copyright (c) 2010 Canada's Michael Smith Genome Sciences Centre. All rights reserved.
"""

import sys
import os

class pslError(Exception):
    """Simple exception class for parsing errors"""
    def __init__(self, message):
        self.message = message
        print "Parsing error: {0}".format(message)

class psl_record:
    def __init__(self, line):
        """Parse a single line of psl input"""
        
        c = line.split()

        # Return nothing for a header line
        try:
            int(c[0])
        except ValueError, e:
            self.header_line = True
            return
        except IndexError, e:
            self.header_line = True
            return    
            
        # Check for the right number of columns
        if not len(c) == 21:
            raise PileupError, "improper number of columns - expecting 21, found {0}\n{1}".format(len(c), line)
        
        self.header_line = False    
        self.match_length = int(c[0])
        self.num_mismatch = int(c[1])
        self.num_rep_match = int(c[2])
        self.num_N = int(c[3])
        self.num_q_inserts = int(c[4])
        self.num_q_insert_bases = int(c[5])
        self.num_t_inserts = int(c[6])
        self.num_t_insert_bases =  int(c[7])
        self.strand = c[8]
        self.q_name = c[9]
        self.q_size = int(c[10])
        self.q_start = int(c[11])
        self.q_end = int(c[12])
        self.t_name = c[13]
        self.t_size = int(c[14])
        self.t_start = int(c[15])
        self.t_end = int(c[16])
        self.blockcount = int(c[17])
        self.blocksizes = c[18]
        self.q_starts = c[19]
        self.t_starts = c[20]
    
        # Convenience fields
        self.match_proportion = float(self.match_length) / float(self.q_size)
    
    def __repr__(self):
        return "<psl_record: {0} hits {1}>".format(self.q_name, self.t_name)
    
    def __str__(self):
        return "\n\t{0}bp hit from query {1} ({2}bp) to target {3} ({4}bp) \
            \n\t{5}-{6} on query to {7}-{8} on target {9} strand \
            \n\t{10} mismatch base(s) in {11} block(s)".format(
            self.match_length, self.q_name, self.q_size, self.t_name, self.t_size,
            self.q_start, self.q_end, self.t_start, self.t_end, self.strand,
            self.num_mismatch, self.blockcount)
    
    # Utility functions
    def is_full_length_hit(self):
        return True if self.match_length == self.q_size else False
                            
if __name__ == '__main__':
    print "Hey, it's psl_record.py!"
    
    in_handle = open(sys.argv[1],'r')
    
    for line in in_handle:
        
        p = psl_record(line)

        if p.header_line == False:
            print p
    
    in_handle.close()